'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});


var CHUNK_NAMES = new Set();

var MODULE_IDS = new Set();

var IS_TEST = process.env.NODE_ENV === 'test';
var isServer = typeof window === 'undefined' || IS_TEST;
var isWebpack = function isWebpack() {
  return typeof __webpack_require__ !== 'undefined';
};
var babelInterop = function babelInterop(obj) {
  return obj && obj.__esModule ? obj.default : obj;
};

var findExport = function findExport(mod, key) {
  if (typeof key === 'function') {
    return key(mod);
  } else if (key === null) {
    return mod;
  }

  return key ? mod[key] : babelInterop(mod);
};

var tryRequire = function tryRequire(id, key, onLoad) {
  try {
    var mod = requireById(id);

    if (onLoad) {
      onLoad(mod);
    }

    return findExport(mod, key);
  } catch (err) {}

  return null;
};

var requireById = function requireById(id) {
  if (!isWebpack() && typeof id === 'string') {
    return module.require(id);
  }

  return __webpack_require__(id);
};

exports.default = function (importAsync, options) {
  var resolve = options.resolve,
      chunkName = options.chunkName,
      path = options.path,
      key = options.key,
      _options$timeout = options.timeout,
      timeout = _options$timeout === undefined ? 15000 : _options$timeout,
      onLoad = options.onLoad;


  var mod = void 0;
  var weakId = void 0;
  var prom = void 0;
  var timer = void 0;

  var requireSync = function requireSync() {
    if (!mod) {
      if (!isWebpack() && path) {
        mod = tryRequire(path, key, onLoad);
      } else if (isWebpack() && resolve) {
        weakId = typeof resolve === 'string' ? resolve : resolve();

        if (__webpack_modules__[weakId]) {
          mod = tryRequire(weakId, key, onLoad);
        }
      }
    }

    return mod;
  };

  var requireAsync = function requireAsync() {
    if (mod) {
      return Promise.resolve(mod);
    }

    if (!prom) {
      prom = new Promise(function (resolve, reject) {
        if (timeout) {
          timer = setTimeout(function () {
            reject(new Error('timeout exceeded'));
          }, timeout);
        }

        var resolveImport = function resolveImport(error, m) {
          clearTimeout(timer);

          if (error) {
            return reject(error);
          }

          if (onLoad && m) {
            onLoad(m);
          }

          mod = findExport(m, key);

          if (mod) {
            return resolve(mod);
          }

          reject(new Error('export not found'));
        };

        var request = typeof importAsync === 'function' ? importAsync(resolveImport) : importAsync;

        // if importAsync doesn't return a promise, it must call resolveImport
        // itself. Most common is the promise implementation below.
        if (!request || !request.then) {
          return;
        }

        request.then(function (m) {
          return resolveImport(null, m);
        }).catch(function (error) {
          clearTimeout(timer);
          reject(error);
        });
      });
    }

    return prom;
  };

  var addModule = function addModule() {
    if (isServer) {
      if (chunkName) {
        CHUNK_NAMES.add(chunkName);
      }

      // just fill both sets so `flushModuleIds` continues to work,
      // even if you decided to start providing chunk names. It's
      // a small array of 3-20 chunk names on average anyway. Users
      // can flush/clear both sets if they feel they need to.
      if (isWebpack() && weakId) {
        MODULE_IDS.add(weakId);
      } else if (!isWebpack() && path) {
        MODULE_IDS.add(path);
      }
    }
  };

  return {
    requireSync: requireSync,
    requireAsync: requireAsync,
    addModule: addModule,
    mod: requireSync()
  };
};

var flushChunkNames = exports.flushChunkNames = function flushChunkNames() {
  var chunks = Array.from(CHUNK_NAMES);
  CHUNK_NAMES.clear();
  return chunks;
};

var flushModuleIds = exports.flushModuleIds = function flushModuleIds() {
  var ids = Array.from(MODULE_IDS);
  MODULE_IDS.clear();
  return ids;
};